@@ -17,7 +17,8 @@
 
 """Python ecosystem specific routines."""
 
-
+from .digests_fetcher import DigestsFetcherBase
+from .digests_fetcher import PythonDigestsFetcher
 from .packages import Packages
 from .package_version import PackageVersion
 from .pipfile import Pipfile
@@ -27,5 +28,5 @@ from .project import Project
 from .source import Source
 
 
-__version__ = '0.3.1'
+__version__ = "0.4.0"
 __author__ = 'Fridolin Pokorny <fridolin@redhat.com>'
