@@ -29,6 +29,8 @@ from thoth.common import cwd
 from thoth.analyzer import run_command
 from thoth.analyzer import CommandError
 
+from .digests_fetcher import DigestsFetcherBase
+from .digests_fetcher import PythonDigestsFetcher
 from .pipfile import Pipfile
 from .pipfile import PipfileLock
 from .pipfile import PipfileMeta
@@ -108,7 +110,9 @@ class Project:
         if packages_locked:
             pipfile_lock = PipfileLock.from_package_versions(pipfile, packages_locked, meta=meta)
 
-        return cls(pipfile, pipfile_lock)
+        instance = cls(pipfile, pipfile_lock)
+        instance.sanitize_source_indexes()
+        return instance
 
     def to_dict(self):
         """Create a dictionary representation of this project."""
@@ -271,28 +275,17 @@ class Project:
 
         return report
 
-    def _index_scan(self) -> typing.Tuple[list, dict]:
+    def _index_scan(self, digests_fetcher: DigestsFetcherBase) -> typing.Tuple[list, dict]:
         """Generate full report for packages given the sources configured."""
         report = {}
         findings = []
         for package_version in chain(self.pipfile_lock.packages, self.pipfile_lock.dev_packages):
             if package_version.name in report:
                 # TODO: can we have the same package in dev packages and packages?
-                raise InternalError(f"Package {package_version.name} already present in the report")
-
-            index_report = {}
-            for source in self.pipfile.meta.sources.values():
-                try:
-                    index_report[source.name] = source.get_package_hashes(
-                        package_version.name,
-                        package_version.locked_version
-                    )
-                except NotFound as exc:
-                    _LOGGER.debug(
-                        f"Package {package_version.name} in version {package_version.version} not "
-                        f"found on index {source.name}: {str(exc)}"
-                    )
+                _LOGGER.warning(f"Package {package_version.name} already present in the report")
+                continue
 
+            index_report = digests_fetcher.fetch_digests(package_version.name, package_version.locked_version)
             findings.extend(self._check_scan(package_version, index_report))
             report[package_version.name] = index_report
 
@@ -330,18 +323,18 @@ class Project:
         if package_version.index:
             source = package_version.index.to_dict()
 
-        if package_version.index and index_report.get(package_version.index.name) and len(hashes) > 1:
+        if package_version.index and index_report.get(package_version.index.url) and len(hashes) > 1:
             # Is installed from different source - which one?
             used_package_version_hashes = set(h[len('sha256:'):] for h in package_version.hashes)
-            configured_index_hashes = set(h['sha256'] for h in index_report[package_version.index.name])
+            configured_index_hashes = set(h['sha256'] for h in index_report[package_version.index.url])
 
             # Find other sources from which artifacts can be installed.
             other_sources = {}
             for artifact_hash in package_version.hashes:
                 artifact_hash = artifact_hash[len('sha256:'):]  # Remove pipenv-specific hash formatting.
 
-                for index_name, index_info in index_report.items():
-                    if index_name == package_version.index.name:
+                for index_url, index_info in index_report.items():
+                    if index_url == package_version.index.url:
                         # Skip index that is assigned to the package, we are inspecting from the other sources.
                         continue
 
@@ -353,10 +346,10 @@ class Project:
                     if not artifact_entry:
                         continue
 
-                    if index_name not in other_sources:
-                        other_sources[index_name] = []
+                    if index_url not in other_sources:
+                        other_sources[index_url] = []
 
-                    other_sources[index_name].extend(artifact_entry)
+                    other_sources[index_url].extend(artifact_entry)
 
             if not set(used_package_version_hashes).issubset(set(configured_index_hashes)):
                 # Source is different from the configured one.
@@ -388,14 +381,14 @@ class Project:
                     'sources': other_sources
                 })
 
-        if package_version.index and not index_report.get(package_version.index.name):
+        if package_version.index and not index_report.get(package_version.index.url):
             # Configured index does not provide the given package.
             scan_report.append({
                 'type': 'ERROR',
                 'id': 'MISSING-PACKAGE',
                 'justification': f'Source index {package_version.index.name!r} explicitly '
-                                 f'assigned to package {package_version.name!r} but package '
-                                 f'was not found on the given index - was it removed?',
+                                 f'assigned to package {package_version.name!r} but there was not found '
+                                 f'any record for the given package',
                 'source': source,
                 'package_locked': package_version.to_pipfile_lock(),
                 'package_name': package_version.name,
@@ -406,14 +399,14 @@ class Project:
         # Changed hashes?
         for digest in package_version.hashes:
             digest = digest[len('sha256:'):]
-            for index_name, index_info in index_report.items():
+            for index_info in index_report.values():
                 if any(item['sha256'] == digest for item in index_info):
                     break
             else:
                 scan_report.append({
                     'type': 'ERROR',
                     'id': 'INVALID-ARTIFACT-HASH',
-                    'justification': 'Artifact stated in lock was not found on index - was it removed?',
+                    'justification': 'Hash for the given artifact was not found',
                     'source': source,
                     'package_locked': package_version.to_pipfile_lock(),
                     'package_name': package_version.name,
@@ -423,12 +416,19 @@ class Project:
 
         return scan_report
 
-    def check_provenance(self, whitelisted_sources: list = None) -> dict:
+    def check_provenance(self, whitelisted_sources: list = None, digests_fetcher: DigestsFetcherBase = None) -> dict:
         """Check provenance/origin of packages that are stated in the project."""
-        findings, _ = self._index_scan()
+        digests_fetcher = digests_fetcher or PythonDigestsFetcher(list(self.pipfile.meta.sources.values()))
+        findings, _ = self._index_scan(digests_fetcher)
         findings.extend(self._check_sources(whitelisted_sources))
         return findings
 
+    def sanitize_source_indexes(self):
+        """Make sure all the indexes are correctly propagated to Pipfile and Pipfile.lock metadata."""
+        self.pipfile.sanitize_source_indexes()
+        if self.pipfile_lock:
+            self.pipenv_lock.sanitize_source_indexes()
+
     def add_source(self, url: str, verify_ssl: bool = True, name: str = None,
                    warehouse: bool = False, warehouse_api_url: str = None) -> Source:
         """Add a package source (index) to the project."""
